@@ -50,7 +50,7 @@ class BaseGravity(CountModel):
                       n x p; p attributes for each destination of n flows;
                       default is None
     constant        : boolean
-                      True to include intercept in model; false by default
+                      True to include intercept in model; True by default
     framework       : string
                       estimation technique; currently only 'GLM' is avaialble
     Quasi           : boolean
@@ -92,7 +92,7 @@ class BaseGravity(CountModel):
     dv              : array
                       n x p(d); p attributes for each destination of n flows
     constant        : boolean
-                      True to include intercept in model; false by default
+                      True to include intercept in model; True by default
     y               : array
                       n x 1; dependent variable used in estimation including any
                       transformations
@@ -122,7 +122,7 @@ class BaseGravity(CountModel):
                       value of the loglikelihood function evaluated with only an
                       intercept; see family.py for distribution-specific
                       loglikelihoods
-    aic             : float
+    AIC             : float
                       Akaike information criterion
     D2              : float
                       percentage of explained deviance
@@ -149,11 +149,11 @@ class BaseGravity(CountModel):
     >>> flows = np.array(db.by_col('count')).reshape((-1,1))
     >>> model = BaseGravity(flows, cost)
     >>> model.params
-    array([ 0.92860101])
+    array([ 17.84839637,  -1.68325787])
 
     """
     def __init__(self, flows, cost, cost_func='pow', o_vars=None, d_vars=None,
-            origins=None, destinations=None, constant=False, framework='GLM',
+            origins=None, destinations=None, constant=True, framework='GLM',
             SF=None, CD=None, Lag=None, Quasi=False):
         n = User.check_arrays(flows, cost)
         #User.check_y(flows, n)
@@ -165,12 +165,16 @@ class BaseGravity(CountModel):
         if type(cost_func) == str:
             if cost_func.lower() == 'pow':
                 self.cf = np.log
+                if (self.c==0).any():
+                    raise ValueError("Zero values detected: cost function 'pow'"
+                            "requires the logarithm of the cost variable which"
+                            "is undefined at 0")
             elif cost_func.lower() == 'exp':
                 self.cf = lambda x: x*1.0
         elif (type(cost_func) == FunctionType) | (type(cost_func) == np.ufunc):
             self.cf = cost_func
         else:
-            raise ValueError("cost_func must be 'exp', 'pow' or a valid"
+            raise ValueError("cost_func must be 'exp', 'pow' or a valid "
             " function that has a scalar as a input and output")
 
         y = np.reshape(self.f, (-1,1))
@@ -178,28 +182,46 @@ class BaseGravity(CountModel):
             X = np.empty((self.n, 0))
         else:
             X = sp.csr_matrix((self.n, 1))
-        if isinstance(self, Attraction) | isinstance(self, Doubly):
-            d_dummies = spcategorical(destinations.flatten())
-            X = sphstack(X, d_dummies, array_out=False)
         if isinstance(self, Production) | isinstance(self, Doubly):
             o_dummies = spcategorical(origins.flatten())
+            if constant:
+                o_dummies = o_dummies[:,1:]
             X = sphstack(X, o_dummies, array_out=False)
-        if isinstance(self, Doubly):
-            X = X[:,1:]
+        if isinstance(self, Attraction) | isinstance(self, Doubly):
+            d_dummies = spcategorical(destinations.flatten())
+            if constant | isinstance(self, Doubly):
+                d_dummies = d_dummies[:,1:]
+            X = sphstack(X, d_dummies, array_out=False)
         if self.ov is not None:	
             if isinstance(self, Gravity):
                 for each in range(self.ov.shape[1]):
+                    if (self.ov[:,each] == 0).any(): 
+                    	raise ValueError("Zero values detected in column %s "
+                                "of origin variables, which are undefined for "
+                                "Poisson log-linear spatial interaction models" % each)
                     X = np.hstack((X, np.log(np.reshape(self.ov[:,each], (-1,1)))))
             else:
                 for each in range(self.ov.shape[1]):
+                    if (self.ov[:,each] == 0).any(): 
+                    	raise ValueError("Zero values detected in column %s "
+                                "of origin variables, which are undefined for "
+                                "Poisson log-linear spatial interaction models" % each)
                     ov = sp.csr_matrix(np.log(np.reshape(self.ov[:,each], ((-1,1)))))
                     X = sphstack(X, ov, array_out=False)
         if self.dv is not None:    	
             if isinstance(self, Gravity):
                 for each in range(self.dv.shape[1]):
+                    if (self.dv[:,each] == 0).any(): 
+                    	raise ValueError("Zero values detected in column %s "
+                                "of destination variables, which are undefined for "
+                                "Poisson log-linear spatial interaction models" % each)
                     X = np.hstack((X, np.log(np.reshape(self.dv[:,each], (-1,1)))))
             else:
                 for each in range(self.dv.shape[1]):
+                    if (self.dv[:,each] == 0).any(): 
+                    	raise ValueError("Zero values detected in column %s "
+                                "of destination variables, which are undefined for "
+                                "Poisson log-linear spatial interaction models" % each)
                     dv = sp.csr_matrix(np.log(np.reshape(self.dv[:,each], ((-1,1)))))
                     X = sphstack(X, dv, array_out=False)
         if isinstance(self, Gravity):
@@ -236,7 +258,7 @@ class BaseGravity(CountModel):
         self.resid_dev = results.resid_dev
         self.llf = results.llf
         self.llnull = results.llnull
-        self.aic = results.aic
+        self.AIC = results.AIC
         self.k = results.k
         self.D2 = results.D2
         self.adj_D2 = results.adj_D2
@@ -283,7 +305,7 @@ class Gravity(BaseGravity):
                       n x p; p attributes for each destination of n flows;
                       default is None
     constant        : boolean
-                      True to include intercept in model; false by default
+                      True to include intercept in model; True by default
     framework       : string
                       estimation technique; currently only 'GLM' is avaialble
     Quasi           : boolean
@@ -324,7 +346,7 @@ class Gravity(BaseGravity):
     dv              : array 
                       n x p(d); p attributes for each destination of n flows
     constant        : boolean
-                      True to include intercept in model; false by default
+                      True to include intercept in model; True by default
     y               : array
                       n x 1; dependent variable used in estimation including any
                       transformations
@@ -354,7 +376,7 @@ class Gravity(BaseGravity):
                       value of the loglikelihood function evaluated with only an
                       intercept; see family.py for distribution-specific
                       loglikelihoods
-    aic             : float 
+    AIC             : float 
                       Akaike information criterion
     D2              : float
                       percentage of explained deviance
@@ -383,11 +405,12 @@ class Gravity(BaseGravity):
     >>> d_cap = np.array(db.by_col('d_cap')).reshape((-1,1))
     >>> model = Gravity(flows, o_cap, d_cap, cost, 'exp')
     >>> model.params
-    array([ 0.87911778,  0.71080687, -0.00194626])
+    array([  3.80050153e+00,   5.54103854e-01,   3.94282921e-01,
+            -2.27091686e-03])
     
     """
     def __init__(self, flows, o_vars, d_vars, cost,
-            cost_func, constant=False, framework='GLM', SF=None, CD=None,
+            cost_func, constant=True, framework='GLM', SF=None, CD=None,
             Lag=None, Quasi=False):
         self.f = np.reshape(flows, (-1,1))
         if len(o_vars.shape) > 1:
@@ -433,7 +456,7 @@ class Gravity(BaseGravity):
         """
         results = {}
         covs = self.ov.shape[1] + self.dv.shape[1] + 1
-        results['aic'] = []
+        results['AIC'] = []
         results['deviance'] = []
         results['pseudoR2'] = []
         results['adj_pseudoR2'] = []
@@ -451,8 +474,9 @@ class Gravity(BaseGravity):
             o_vars = self.ov[subset.reshape(self.ov.shape[0]),:]
             d_vars = self.dv[subset.reshape(self.dv.shape[0]),:]
             dij = self.reshape(self.c[subset])
-            model = Gravity(f, o_vars, d_vars, dij, self.cf)
-            results['aic'].append(model.aic)
+            model = Gravity(f, o_vars, d_vars, dij, self.cf,
+                    constant=False)
+            results['AIC'].append(model.AIC)
             results['deviance'].append(model.deviance)
             results['pseudoR2'].append(model.pseudoR2)
             results['adj_pseudoR2'].append(model.adj_pseudoR2)
@@ -488,7 +512,7 @@ class Production(BaseGravity):
                       n x p; p attributes for each destination of n flows;
                       default is None
     constant        : boolean
-                      True to include intercept in model; false by default
+                      True to include intercept in model; True by default
     framework       : string
                       estimation technique; currently only 'GLM' is avaialble
     Quasi           : boolean
@@ -529,7 +553,7 @@ class Production(BaseGravity):
     dv              : array 
                       n x p; p attributes for each destination of n flows
     constant        : boolean
-                      True to include intercept in model; false by default
+                      True to include intercept in model; True by default
     y               : array
                       n x 1; dependent variable used in estimation including any
                       transformations
@@ -559,7 +583,7 @@ class Production(BaseGravity):
                       value of the loglikelihood function evaluated with only an
                       intercept; see family.py for distribution-specific
                       loglikelihoods
-    aic             : float 
+    AIC             : float 
                       Akaike information criterion
     D2              : float
                       percentage of explained deviance
@@ -589,11 +613,10 @@ class Production(BaseGravity):
     >>> d_cap = np.array(db.by_col('d_cap')).reshape((-1,1))
     >>> model = Production(flows, o, d_cap, cost, 'exp')
     >>> model.params[-4:]
-    array([  5.38580065e+00,   5.00216058e+00,   8.55357745e-01,
-            -2.27444394e-03])
+    array([ 1.34721352,  0.96357345,  0.85535775, -0.00227444])
 
     """
-    def __init__(self, flows, origins, d_vars, cost, cost_func, constant=False,
+    def __init__(self, flows, origins, d_vars, cost, cost_func, constant=True,
             framework='GLM', SF=None, CD=None, Lag=None, Quasi=False):
         self.constant = constant
         self.f = self.reshape(flows)
@@ -629,7 +652,7 @@ class Production(BaseGravity):
         """
         results = {}
         covs = self.dv.shape[1] + 1
-        results['aic'] = []
+        results['AIC'] = []
         results['deviance'] = []
         results['pseudoR2'] = []
         results['adj_pseudoR2'] = []
@@ -649,8 +672,8 @@ class Production(BaseGravity):
             o = self.reshape(self.o[subset])
             d_vars = self.dv[subset.reshape(self.dv.shape[0]),:]
             dij = self.reshape(self.c[subset])
-            model = Production(f, o, d_vars, dij, self.cf)
-            results['aic'].append(model.aic)
+            model = Production(f, o, d_vars, dij, self.cf, constant=False)
+            results['AIC'].append(model.AIC)
             results['deviance'].append(model.deviance)
             results['pseudoR2'].append(model.pseudoR2)
             results['adj_pseudoR2'].append(model.adj_pseudoR2)
@@ -686,7 +709,7 @@ class Attraction(BaseGravity):
                       n x p; p attributes for each origin of  n flows; default
                       is None
     constant        : boolean
-                      True to include intercept in model; false by default
+                      True to include intercept in model; True by default
     y               : array
                       n x 1; dependent variable used in estimation including any
                       transformations
@@ -732,7 +755,7 @@ class Attraction(BaseGravity):
     ov              : array
                       n x p; p attributes for each origin of n flows
     constant        : boolean
-                      True to include intercept in model; false by default
+                      True to include intercept in model; True by default
     params          : array
                       n x k, k estimated beta coefficients; k = # of
                       destinations + p + 1
@@ -758,7 +781,7 @@ class Attraction(BaseGravity):
                       value of the loglikelihood function evaluated with only an
                       intercept; see family.py for distribution-specific
                       loglikelihoods
-    aic             : float 
+    AIC             : float 
                       Akaike information criterion
     D2              : float
                       percentage of explained deviance
@@ -787,12 +810,11 @@ class Attraction(BaseGravity):
     >>> o_cap = np.array(db.by_col('o_cap')).reshape((-1,1))
     >>> model = Attraction(flows, d, o_cap, cost, 'exp')
     >>> model.params[-4:]
-    array([  5.23366116e+00,   4.89037868e+00,   8.82909095e-01,
-            -2.29081323e-03])
-
+    array([ 1.21962276,  0.87634028,  0.88290909, -0.00229081])
+    
     """
     def __init__(self, flows, destinations, o_vars, cost, cost_func,
-            constant=False, framework='GLM', SF=None, CD=None, Lag=None,
+            constant=True, framework='GLM', SF=None, CD=None, Lag=None,
             Quasi=False):
         self.f = np.reshape(flows, (-1,1))
         if len(o_vars.shape) > 1:
@@ -825,7 +847,7 @@ class Attraction(BaseGravity):
         """
         results = {}
         covs = self.ov.shape[1] + 1
-        results['aic'] = []
+        results['AIC'] = []
         results['deviance'] = []
         results['pseudoR2'] = []
         results['adj_pseudoR2'] = []
@@ -845,8 +867,8 @@ class Attraction(BaseGravity):
             d = self.reshape(self.d[subset])
             o_vars = self.ov[subset.reshape(self.ov.shape[0]),:]
             dij = self.reshape(self.c[subset])
-            model = Attraction(f, d, o_vars, dij, self.cf)
-            results['aic'].append(model.aic)
+            model = Attraction(f, d, o_vars, dij, self.cf, constant=False)
+            results['AIC'].append(model.AIC)
             results['deviance'].append(model.deviance)
             results['pseudoR2'].append(model.pseudoR2)
             results['adj_pseudoR2'].append(model.adj_pseudoR2)
@@ -883,7 +905,7 @@ class Doubly(BaseGravity):
                       functional form of the cost function;
                       'exp' | 'pow' | custom function
     constant        : boolean
-                      True to include intercept in model; false by default
+                      True to include intercept in model; True by default
     y               : array
                       n x 1; dependent variable used in estimation including any
                       transformations
@@ -929,7 +951,7 @@ class Doubly(BaseGravity):
     d               : array
                       n x 1; index of destination id's
     constant        : boolean
-                      True to include intercept in model; false by default
+                      True to include intercept in model; True by default
     params          : array
                       n x k, estimated beta coefficients; k = # of origins + #
                       of destinations; the first x-1 values
@@ -959,7 +981,7 @@ class Doubly(BaseGravity):
                       value of the loglikelihood function evaluated with only an
                       intercept; see family.py for distribution-specific
                       loglikelihoods
-    aic             : float 
+    AIC             : float 
                       Akaike information criterion
     D2              : float
                       percentage of explained deviance
@@ -992,7 +1014,7 @@ class Doubly(BaseGravity):
 
     """
     def __init__(self, flows, origins, destinations, cost, cost_func,
-            constant=False, framework='GLM', SF=None, CD=None, Lag=None,
+            constant=True, framework='GLM', SF=None, CD=None, Lag=None,
             Quasi=False):
 
         self.f = np.reshape(flows, (-1,1))
